"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports._buildRegex = _buildRegex;
exports.buildPattern = buildPattern;
exports.buildRegex = buildRegex;
var _encoder = require("./encoder/encoder");
var _nodes = require("./utils/nodes");
var _optionalArg = require("./utils/optional-arg");
/**
 * Generate RegExp object from elements.
 *
 * @param elements Single regex element or array of elements
 * @returns
 */

/**
 * Generate RegExp object from elements with passed flags.
 *
 * @param elements Single regex element or array of elements
 * @param flags RegExp flags object
 * @returns RegExp object
 */

function buildRegex(first, second) {
  return _buildRegex(...(0, _optionalArg.optionalFirstArg)(first, second));
}
function _buildRegex(flags, elements) {
  const pattern = (0, _encoder.encodeSequence)((0, _nodes.asNodeArray)(elements)).pattern;
  const flagsString = encodeFlags(flags ?? {});
  return new RegExp(pattern, flagsString);
}

/**
 * Generate regex pattern from elements.
 * @param elements Single regex element or array of elements
 * @returns regex pattern string
 */
function buildPattern(elements) {
  return (0, _encoder.encodeSequence)((0, _nodes.asNodeArray)(elements)).pattern;
}
function encodeFlags(flags) {
  let result = '';
  if (flags.global) result += 'g';
  if (flags.ignoreCase) result += 'i';
  if (flags.multiline) result += 'm';
  if (flags.hasIndices) result += 'd';
  if (flags.sticky) result += 'y';
  return result;
}
//# sourceMappingURL=builders.js.map