import { encodeSequence } from './encoder/encoder';
import { asNodeArray } from './utils/nodes';
import { optionalFirstArg } from './utils/optional-arg';

/**
 * Generate RegExp object from elements.
 *
 * @param elements Single regex element or array of elements
 * @returns
 */

/**
 * Generate RegExp object from elements with passed flags.
 *
 * @param elements Single regex element or array of elements
 * @param flags RegExp flags object
 * @returns RegExp object
 */

export function buildRegex(first, second) {
  return _buildRegex(...optionalFirstArg(first, second));
}
export function _buildRegex(flags, elements) {
  const pattern = encodeSequence(asNodeArray(elements)).pattern;
  const flagsString = encodeFlags(flags ?? {});
  return new RegExp(pattern, flagsString);
}

/**
 * Generate regex pattern from elements.
 * @param elements Single regex element or array of elements
 * @returns regex pattern string
 */
export function buildPattern(elements) {
  return encodeSequence(asNodeArray(elements)).pattern;
}
function encodeFlags(flags) {
  let result = '';
  if (flags.global) result += 'g';
  if (flags.ignoreCase) result += 'i';
  if (flags.multiline) result += 'm';
  if (flags.hasIndices) result += 'd';
  if (flags.sticky) result += 'y';
  return result;
}
//# sourceMappingURL=builders.js.map